/* SPDX-License-Identifier: LGPL-2.1-or-later
 *
 * Frv - Frame vector library
 *
 * Copyright © 2001-2021 Laboratoire d'Annecy de physique des particules - CNRS
 * Copyright © 2019-2021 Institut pluridisciplinaire Hubert CURIEN - CNRS
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

/*-------------------------------------------------------------------*/
/*  FrvCopy.h by B.Mours LAPP (Annecy)         Jun  6, 2012          */
/*-------------------------------------------------------------------*/

#ifndef FRVCOPY
#define FRVCOPY

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif

#include "FrameL.h"

#ifdef __cplusplus
extern "C" {
#endif

FrVect *FrvClone(FrVect *vect1, char* newName);

#define FrvDecimate FrVectDecimate
#define FrvCopyTo   FrVectCopyTo 
#define FrvCopyToD  FrVectCopyToD
#define FrvCopyToF  FrVectCopyToF
#define FrvCopyToI  FrVectCopyToI
#define FrvCopyToS  FrVectCopyToS

#define FrvDecimate FrVectDecimate

FrVect *FrvDecimateD(FrVect *vIn, int nGroup, char *newName);
FrVect *FrvDecimateF(FrVect *vIn, int nGroup, char *newName);
FrVect *FrvDecimateI(FrVect *vIn, int nGroup, char *newName);
FrVect *FrvDecimateS(FrVect *vIn, int nGroup, char *newName);

#ifdef __cplusplus
}
#endif

#endif
